<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Abandoned Cart Email Subscription Functionality
 */

namespace Amasty\AcartSubscriptionFunctionality\Model;

use Amasty\Acart\Model\ConfigProvider as AcartConfigProvider;

class ConfigProvider extends AcartConfigProvider
{
    public const THRESHOLD_REACHING_NOTIFY = 'admin_notifications/threshold_reaching_notify';
    public const THRESHOLD_NOTIFY_QTY = 'admin_notifications/threshold_notify_qty';
    public const THRESHOLD_EMAIL_TEMPLATE = 'admin_notifications/threshold_email_template';
    public const THRESHOLD_EMAIL_RECIPIENT = 'admin_notifications/threshold_admin_email_recipient';
    public const THRESHOLD_SENDER_IDENTITY = 'admin_notifications/threshold_sender_email_identity';

    public function isNotifyThresholdReaching(?int $storeId = null): bool
    {
        return $this->isSetFlag(self::THRESHOLD_REACHING_NOTIFY, $storeId);
    }

    public function getThresholdNotifyQty(?int $storeId = null): int
    {
        return (int)$this->getValue(self::THRESHOLD_NOTIFY_QTY, $storeId);
    }

    public function getThresholdEmailTemplate(?int $storeId = null): string
    {
        return (string)$this->getValue(self::THRESHOLD_EMAIL_TEMPLATE, $storeId);
    }

    public function getThresholdEmailRecipients(?int $storeId = null): array
    {
        $recipientsArray = [];

        if ($recipients = $this->getValue(self::THRESHOLD_EMAIL_RECIPIENT, $storeId)) {
            $recipientsArray = array_filter(array_map('trim', preg_split('/\n|\r\n?/', $recipients)));
        }

        return $recipientsArray;
    }

    public function getThresholdEmailSender(?int $storeId = null): string
    {
        return (string)$this->getValue(self::THRESHOLD_SENDER_IDENTITY, $storeId);
    }
}
