<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Abandoned Cart Email Subscription Functionality
 */

namespace Amasty\AcartSubscriptionFunctionality\Model\CustomCode;

class CodeGenerator
{
    /**
     * @var array
     */
    protected $existingCodes = [];

    /**
     * @var array
     */
    private $templateMasksList = [];

    /**
     * @var CodeTemplateValidator
     */
    private $codeTemplateValidator;

    public function __construct(
        CodeTemplateValidator $codeTemplateValidator
    ) {
        $this->codeTemplateValidator = $codeTemplateValidator;
    }

    public function generateCode(string $template): string
    {
        $this->initializeByTemplate($template);
        $code = $template;
        foreach ($this->templateMasksList[$template] as $templateSymbol) {
            $possibleValues = CodeTemplateValidator::MASK[$templateSymbol];
            $symbol = (string)$possibleValues[array_rand($possibleValues)];
            $code = preg_replace('/' . preg_quote($templateSymbol, '/') . '/', $symbol, $code, 1);
        }

        if ($this->isCodeExist($code, $template)) {
            return $this->generateCode($template);
        }
        $this->existingCodes[$template][] = $code;

        return $code;
    }

    public function getTemplateAvailableCodesQty(string $template): float
    {
        $this->initializeByTemplate($template);

        return (float)($this->codeTemplateValidator->getTemplateMaxCodesQty($template)
            - count($this->existingCodes[$template]));
    }

    private function initializeByTemplate(string $template)
    {
        if (!array_key_exists($template, $this->existingCodes)) {
            $this->existingCodes[$template] = $this->codeTemplateValidator->getCodesByTemplate($template);
        }
        if (!array_key_exists($template, $this->templateMasksList)) {
            $this->templateMasksList[$template] = $this->codeTemplateValidator->getTemplateMasksList($template);
        }
    }

    private function isCodeExist(string $code, string $template): bool
    {
        return (!empty($this->existingCodes[$template]) && in_array($code, $this->existingCodes[$template]));
    }

    public function clearExistingCodeStorage()
    {
        $this->existingCodes = [];
    }
}
