<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Abandoned Cart Email Subscription Functionality
 */

namespace Amasty\AcartSubscriptionFunctionality\Model\CustomCode;

use Amasty\AcartSubscriptionFunctionality\Model\ResourceModel\History\GetCouponsByTemplate;

class CodeTemplateValidator
{
    public const MASK = [
        // no "0" and "1" as they are confusing
        '{D}' => [2, 3, 4, 5, 6, 7, 8, 9],
        // no I, Q and O as they are confusing
        '{L}' => ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'J', 'K', 'L', 'M',
                  'N', 'P', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z']
    ];

    /**
     * @var GetCouponsByTemplate
     */
    private $getCouponsByTemplate;

    public function __construct(
        GetCouponsByTemplate $getCouponsByTemplate
    ) {
        $this->getCouponsByTemplate = $getCouponsByTemplate;
    }

    public function getCodesByTemplate(string $template): array
    {
        $dbTemplate = $this->getDbTemplate($template);

        return $this->getCouponsByTemplate->get($dbTemplate);
    }

    public function getDbTemplate(string $template): string
    {
        $dbTemplate = $template;
        foreach (self::MASK as $placeholder => $values) {
            $dbTemplate = str_replace($placeholder, "_", $dbTemplate);
        }

        return $dbTemplate;
    }

    public function getTemplateMasksList(string $template): array
    {
        $templateMasksList = [];
        $masks = array_map(
            static function ($value) {
                return preg_quote($value, '/');
            },
            array_keys(self::MASK)
        );

        $regExpTemplate = implode('|', $masks);
        if (preg_match_all('/' . $regExpTemplate . '/', $template, $matches)) {
            $templateMasksList = $matches[0];
        }

        return $templateMasksList;
    }

    public function getTemplateMaxCodesQty(string $template): float
    {
        $maxQty = 1;

        foreach (self::MASK as $placeholder => $values) {
            $allValuesCount = count($values);
            $templateValuesCount = substr_count($template, $placeholder);
            $maxQty *= $allValuesCount ** $templateValuesCount;
        }

        return (float)$maxQty;
    }
}
