<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Abandoned Cart Email Subscription Functionality
 */

namespace Amasty\AcartSubscriptionFunctionality\Model\EmailSender;

use Amasty\Acart\Api\Data\RuleInterface;
use Amasty\AcartSubscriptionFunctionality\Model\ConfigProvider;
use Magento\Framework\App\Area;
use Magento\Framework\App\State as AppState;
use Magento\Framework\Mail\Template\TransportBuilder;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;

class ReachingThresholdSender
{
    /**
     * @var ConfigProvider
     */
    private $configProvider;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var TransportBuilder
     */
    private $transportBuilder;

    /**
     * @var AppState
     */
    private $appState;

    public function __construct(
        ConfigProvider $configProvider,
        LoggerInterface $logger,
        StoreManagerInterface $storeManager,
        TransportBuilder $transportBuilder,
        AppState $appState
    ) {
        $this->configProvider = $configProvider;
        $this->logger = $logger;
        $this->storeManager = $storeManager;
        $this->transportBuilder = $transportBuilder;
        $this->appState = $appState;
    }

    public function execute(RuleInterface $rule, int $qty): void
    {
        $storeId = (int)$this->storeManager->getStore()->getId();
        if (!$this->configProvider->isNotifyThresholdReaching($storeId)) {
            return;
        }
        try {
            $sendTo = $this->configProvider->getThresholdEmailRecipients($storeId);
            $templateIdentifier = $this->configProvider->getThresholdEmailTemplate($storeId);
            $sender = $this->configProvider->getThresholdEmailSender($storeId);
            foreach ($sendTo as $receiver) {
                $this->transportBuilder->setTemplateIdentifier($templateIdentifier)
                    ->setTemplateOptions(['area' => Area::AREA_FRONTEND, 'store' => $storeId])
                    ->setTemplateVars(['rule_name' => $rule->getName(), 'qty' => $qty])
                    ->setFromByScope($sender, $storeId);

                $this->transportBuilder->addTo($receiver);
                $transport = $this->appState->emulateAreaCode(
                    Area::AREA_FRONTEND,
                    [$this->transportBuilder, 'getTransport']
                );
                $transport->sendMessage();
            }
        } catch (\Exception $e) {
            $this->logger->critical($e);
        }
    }
}
