<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Abandoned Cart Email Subscription Functionality
 */

namespace Amasty\AcartSubscriptionFunctionality\Model\Rule\Handlers;

use Amasty\Acart\Api\Data\RuleInterface;
use Amasty\AcartSubscriptionFunctionality\Api\Data\RuleCouponCodeInterfaceFactory;
use Amasty\AcartSubscriptionFunctionality\Model\Rule\ResourceModel\RuleCouponCode;
use Amasty\AcartSubscriptionFunctionality\Model\Rule\RuleCouponCode as RuleCouponCodeModel;

class LoadHandler
{
    /**
     * @var RuleCouponCode
     */
    private $resource;

    /**
     * @var RuleCouponCodeInterfaceFactory
     */
    private $dataFactory;

    public function __construct(
        RuleCouponCode $resource,
        RuleCouponCodeInterfaceFactory $dataFactory
    ) {
        $this->resource = $resource;
        $this->dataFactory = $dataFactory;
    }

    public function execute(RuleInterface $rule): void
    {
        $extension = $rule->getExtensionAttributes();
        if ($extension->getRuleCouponCode()) {
            return;
        }
        $data = $this->dataFactory->create();
        $this->resource->load($data, $rule->getRuleId(), RuleCouponCodeModel::RULE_ID);

        $extension->setRuleCouponCode($data);
        $rule->setExtensionAttributes($extension);
    }
}
