<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Abandoned Cart Email Subscription Functionality
 */

namespace Amasty\AcartSubscriptionFunctionality\Model\Rule\Relation;

use Amasty\AcartSubscriptionFunctionality\Api\Data\RuleCouponCodeInterfaceFactory;
use Amasty\AcartSubscriptionFunctionality\Model\Rule\ResourceModel\RuleCouponCode;
use Amasty\AcartSubscriptionFunctionality\Model\Rule\RuleCouponCode as RuleCouponCodeModel;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Model\AbstractModel;
use Magento\Framework\Model\ResourceModel\Db\VersionControl\RelationInterface;

class CouponCodeProcessor implements RelationInterface
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * @var RuleCouponCode
     */
    private $ruleCouponCode;

    /**
     * @var RuleCouponCodeInterfaceFactory
     */
    private $dataFactory;

    public function __construct(
        ResourceConnection $resourceConnection,
        RuleCouponCode $ruleCouponCode,
        RuleCouponCodeInterfaceFactory $dataFactory
    ) {
        $this->resourceConnection = $resourceConnection;
        $this->ruleCouponCode = $ruleCouponCode;
        $this->dataFactory = $dataFactory;
    }

    public function processRelation(AbstractModel $rule)
    {
        $template = $rule->getData('template');
        $couponCodeData = $this->dataFactory->create();
        $this->ruleCouponCode->load($couponCodeData, $rule->getRuleId(), RuleCouponCodeModel::RULE_ID);
        $couponCodeData->setTemplate($template);
        $couponCodeData->setRuleId((int)$rule->getRuleId());
        $this->resourceConnection->getConnection()->insertOnDuplicate(
            $this->resourceConnection->getTableName(RuleCouponCode::TABLE_NAME),
            $couponCodeData->getData()
        );
    }
}
