<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Abandoned Cart Email Subscription Functionality
 */

namespace Amasty\AcartSubscriptionFunctionality\Observer;

use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Stdlib\DateTime\DateTime;

class AddTemplateVars implements ObserverInterface
{
    /**
     * @var DateTime
     */
    private $dateTime;

    public function __construct(
        DateTime $dateTime
    ) {
        $this->dateTime = $dateTime;
    }

    public function execute(Observer $observer): void
    {
        if (($acartRule = $observer->getEvent()->getRule())
            && ($history = $observer->getEvent()->getHistory()) && $vars = $observer->getEvent()->getVars()
        ) {
            $historyDate = $history->getSalesRuleCouponExpirationDate();
            if (empty($historyDate)) {
                return;
            }
            if ($dateFormat = $acartRule->getDateFormat()) {
                $vars->setData('couponExpirationDate', $this->dateTime->date($dateFormat, $historyDate));
            }
        }
    }
}
