<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Abandoned Cart Email Subscription Functionality
 */

namespace Amasty\AcartSubscriptionFunctionality\Observer\SalesRule;

use Amasty\Acart\Api\Data\RuleInterface;
use Amasty\AcartSubscriptionFunctionality\Model\CustomCode\CodeGenerator;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\SalesRule\Model\Rule;

class PrepareSalesRule implements ObserverInterface
{
    /**
     * @var CodeGenerator
     */
    private $codeGenerator;

    public function __construct(
        CodeGenerator $codeGenerator
    ) {
        $this->codeGenerator = $codeGenerator;
    }

    public function execute(Observer $observer): void
    {
        if (($rule = $observer->getEvent()->getRule()) && ($acartRule = $observer->getEvent()->getAcartRule())
            && $acartRule->getExtensionAttributes()->getRuleCouponCode()
            && $acartRule->getExtensionAttributes()->getRuleCouponCode()->getTemplate()
        ) {
            /** @var RuleInterface $acartRule */
            $template = $acartRule->getExtensionAttributes()->getRuleCouponCode()->getTemplate();
            if (!empty($template)) {
                $this->processCouponCode($template, $rule);
            }
        }
    }

    private function processCouponCode(string $template, Rule $rule): void
    {
        $availableQty = (int)$this->codeGenerator->getTemplateAvailableCodesQty($template);
        if ($availableQty > 0) {
            $customCouponCode = $this->codeGenerator->generateCode($template);
            $rule->setCouponCode($customCouponCode);
        }
    }
}
