<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Abandoned Cart Email Subscription Functionality
 */

namespace Amasty\AcartSubscriptionFunctionality\Plugin\Controller\Adminhtml\Rule\Edit;

use Amasty\Acart\Api\RuleRepositoryInterface;
use Amasty\Acart\Controller\Adminhtml\Rule\Edit;
use Amasty\AcartSubscriptionFunctionality\Model\ConfigProvider;
use Amasty\AcartSubscriptionFunctionality\Model\CustomCode\CodeGenerator;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Exception\NotFoundException;
use Magento\Framework\Message\ManagerInterface as MessageManagerInterface;

class ShowMessagePlugin
{
    /**
     * @var MessageManagerInterface
     */
    private $manager;

    /**
     * @var ConfigProvider
     */
    private $configProvider;

    /**
     * @var RequestInterface
     */
    private $request;

    /**
     * @var RuleRepositoryInterface
     */
    private $ruleRepository;

    /**
     * @var CodeGenerator
     */
    private $codeGenerator;

    public function __construct(
        MessageManagerInterface $manager,
        ConfigProvider $configProvider,
        RequestInterface $request,
        RuleRepositoryInterface $ruleRepository,
        CodeGenerator $codeGenerator
    ) {
        $this->manager = $manager;
        $this->configProvider = $configProvider;
        $this->request = $request;
        $this->ruleRepository = $ruleRepository;
        $this->codeGenerator = $codeGenerator;
    }

    public function beforeExecute(Edit $subject): void
    {
        if (!($ruleId = (int)$this->request->getParam('id'))) {
            return;
        }
        try {
            $rule = $this->ruleRepository->get($ruleId);
        } catch (NotFoundException $exception) {
            return;
        }
        if ($rule->getExtensionAttributes()
            && $rule->getExtensionAttributes()->getRuleCouponCode()
            && $template = $rule->getExtensionAttributes()->getRuleCouponCode()->getTemplate()
        ) {
            $notifyQty = $this->configProvider->getThresholdNotifyQty();
            $availableQty = (int)$this->codeGenerator->getTemplateAvailableCodesQty($template);
            if ($availableQty === 0) {
                $this->manager->addWarningMessage(
                    __(
                        'No more unique coupons left available for this campaign. ' .
                        'The mailing continues with the standard system coupon codes.'
                    )
                );
            } elseif ($availableQty < $notifyQty) {
                $this->manager->addWarningMessage(
                    __(
                        '%1 unique coupons left available for this campaign. ' .
                        'Please change your coupons custom name parameters otherwise the ' .
                        'mailing will proceed with the standard system coupon codes.',
                        $availableQty
                    )
                );
            }
        }
    }
}
