<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Abandoned Cart Email Subscription Functionality
 */

namespace Amasty\AcartSubscriptionFunctionality\Plugin\Model\RuleQuoteFromRuleAndQuoteFactory;

use Amasty\Acart\Api\Data\RuleQuoteInterface;
use Amasty\Acart\Model\Rule;
use Amasty\Acart\Model\RuleQuoteFromRuleAndQuoteFactory;
use Amasty\AcartSubscriptionFunctionality\Model\ConfigProvider;
use Amasty\AcartSubscriptionFunctionality\Model\CustomCode\CodeGenerator;
use Amasty\AcartSubscriptionFunctionality\Model\EmailSender\ReachingThresholdSender;

class ThresholdNotifyPlugin
{
    /**
     * @var ReachingThresholdSender
     */
    private $reachingThresholdSender;

    /**
     * @var ConfigProvider
     */
    private $configProvider;

    /**
     * @var CodeGenerator
     */
    private $codeGenerator;

    public function __construct(
        ReachingThresholdSender $reachingThresholdSender,
        ConfigProvider $configProvider,
        CodeGenerator $codeGenerator
    ) {
        $this->reachingThresholdSender = $reachingThresholdSender;
        $this->configProvider = $configProvider;
        $this->codeGenerator = $codeGenerator;
    }

    public function afterCreate(
        RuleQuoteFromRuleAndQuoteFactory $subject,
        RuleQuoteInterface $ruleQuote,
        Rule $rule
    ): RuleQuoteInterface {
        if ($rule->getExtensionAttributes()->getRuleCouponCode()
            && $template = $rule->getExtensionAttributes()->getRuleCouponCode()->getTemplate()
        ) {
            $this->codeGenerator->clearExistingCodeStorage();
            $availableQty = (int)$this->codeGenerator->getTemplateAvailableCodesQty($template);
            if ($availableQty === $this->configProvider->getThresholdNotifyQty()) {
                $this->reachingThresholdSender->execute($rule, $availableQty);
            }
        }

        return $ruleQuote;
    }
}
